import argparse

CBOLD = '\33[1m'
CEND = '\33[0m'
CRED = '\33[31m'
CGREEN = '\33[32m'
CYELLOW = '\33[33m'

def get_parser():
    parser = argparse.ArgumentParser(
        prog="sragetter",
        description='Tool for download the Sequencing Reads Archive data through ENA by using aspera',
        formatter_class=argparse.RawTextHelpFormatter,
        epilog=f"""
{CBOLD}Usage{CEND}
-----------------
Download with one accession:
    {CRED}$ {CGREEN}sragetter{CEND} {CYELLOW}--accession{CEND} sra_accession {CYELLOW}--ssh-key{CEND} sshkey_path.ssh {CYELLOW}--outdir{CEND} outdir_path 

Download with TXT file containing multiple accessions:
    {CRED}$ {CGREEN}sragetter{CEND} {CYELLOW}--file{CEND} sra_accessions.txt {CYELLOW}--ssh-key{CEND} sshkey_path.ssh {CYELLOW}--outdir{CEND} outdir_path

Runjia Ji, 2023
"""
    )

    parser.add_argument('-v', '--version', action='version', version='%(prog)s 2.0', help='Show SRAdownloader version number and exit')
    group = parser.add_mutually_exclusive_group()
    group.add_argument('-acc', '--accession', type=str, help='SRA data accession')
    group.add_argument('-f', '--file', type=str, help='TXT file with multiple SRA accessions')
    parser.add_argument('-ssh', '--ssh-key', dest='ssh_key', type=str, help="Public key authentication file provided by Aspera command line client download package as the 'asperaweb_id_dsa.openssh' file", required=True)
    parser.add_argument('-o', '--outdir', type=str, help='Path to store the downloaded SRA data', required=True)
    args = parser.parse_args()
    return args



#!/usr/bin/env python3
import logging
import os

logging.basicConfig(
    level=logging.INFO, 
    format='[%(asctime)s] %(levelname)s: %(message)s',
    datefmt="%H:%M:%S"
    )

def main():
    from .getparser import get_parser
    from .getsra import dl

    args = get_parser()
    acc = args.accession
    inputfile = args.file
    sshkey = args.ssh_key
    outdir = args.outdir
    if acc:
        dl(acc=acc, sshkey=sshkey, outdir=outdir)
    elif inputfile:
        if os.path.isfile(inputfile):
            logging.info(f'Input file is {inputfile}')
            with open(inputfile)as f:
                accs=[i.strip() for i in f.readlines()]
            for acc in accs:
                dl(acc=acc, sshkey=sshkey, outdir=outdir)
        else:
            logging.info(f'{inputfile} is not a file. Exiting...')
                
#############################################
if __name__ == '__main__':
    main()


from .getparser import get_parser
import requests
from io import StringIO
import pandas as pd
import math
import os
import logging

logging.basicConfig(
    level=logging.INFO, 
    format='[%(asctime)s] %(levelname)s: %(message)s',
    datefmt="%H:%M:%S"
    )

def dl(acc, sshkey, outdir):
    _s=acc.center(16)
    print(
        f"""
####################
##{_s}##
####################
"""
    )
    if os.path.isdir(outdir):
        logging.info(f"SRA data will be downloaded into {outdir}")
    else:
        logging.critical('An error occurred. Please see the info below')
        raise Exception(f'Sorry, the {outdir} does not exists. Please pass a exist folder to store the SRA data.')
    
    log_file = os.path.join(outdir,'download_records.tsv')
    base_url = 'era-fasp@fasp.sra.ebi.ac.uk:'

    headers = {
        'accept': '*/*',
    }

    params = {
        'accession': acc,
        'result': 'read_run',
    }

    # Getting the accession report and write out to TSV file
    logging.info(f"{acc} file report is being retrieved from ENA using ENA filereport api")
    response = requests.get('https://www.ebi.ac.uk/ena/portal/api/filereport', params=params, headers=headers)
    record = pd.read_csv(StringIO(response.text), sep='\t')
    record['datetime'] = pd.Timestamp.now()

    logging.info(f"Writing {acc} report to download_records.tsv file")
    if not os.path.isfile(log_file):
        record.to_csv(log_file, index=False, sep='\t')
    else:
        records = pd.read_csv(log_file, sep='\t')
        records = pd.concat([records, record])
        records = records.drop_duplicates(subset=['run_accession'], keep='last')
        records.to_csv(log_file, index=False, sep='\t')

    # Getting the fastq ftp urls for Aspera to download fastqs
    fastq_ftp = record.loc[0, 'fastq_ftp']
    if isinstance(fastq_ftp,str):
        fq_urls = fastq_ftp.split(';')
    elif math.isnan(fastq_ftp):# If no fastq ftq urls, warning the user and abort this download
        logging.warning('No fastq ftp url was found')
        logging.warning(f'Downloading aborted. Writing {acc} to failed.txt')
        failed_file = open(os.path.join(outdir, 'failed.txt'), 'a', encoding='utf-8')
        failed_file.write(f'{acc}\n')
        failed_file.close()
        return
    
    md5_value = record.loc[0, 'fastq_md5']
    md5s = md5_value.split(';')

    # Download fastqs and record md5sum hash values
    md5_file = open(os.path.join(outdir, 'fastq.md5'), 'a', encoding='utf-8')
    for _url, md5 in zip(fq_urls, md5s):
        url = base_url + _url.lstrip('ftp.sra.ebi.ac.uk/')
        cmd = f'ascp -QT -l 300m -P 33001 -i "{sshkey}" "{url}" "{outdir}"'
        logging.info(f'Command is: {cmd}')
        logging.info('Downloading has started...')
        return_code = os.system(cmd)
        if return_code == 0:
            filename = _url.split('/')[-1]
            logging.info(f"Writing {filename} md5 hash value to fastq.md5 file")
            md5_file.write(f'{md5}\t{filename}\n')
        else:
            logging.info(f'Something went wrong when downloading {filename}')
            logging.info(f'{filename} downloading failed. Writing {filename} to failed.txt')
            failed_file = open(os.path.join(outdir, 'failed.txt'), 'a', encoding='utf-8')
            failed_file.write(f'{filename}\n')
            failed_file.close()
    logging.info(f'{acc} data successfully downloaded')
    md5_file.close()
    



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()
setup(
    name='AsperaSRAgetter',
    version='2.0',
    description="The AsperaSRAgetter provides a easy way to download sequencing data from ENA by using Aspera.",
    url="https://github.com/RunJiaJi/AsperaSRAgetter",
    author='Runjia Ji',
    author_email='jirunjia@gmail.com',
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "License :: OSI Approved :: MIT License",
        "Operating System :: POSIX :: Linux",
    ],
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=[
        'pandas',
        'requests'
    ],
    entry_points={
        'console_scripts':[
            'sragetter = AsperaSRAgetter:main',
        ]
    },
)